CREATE TABLE transactions (
  id        INT                 NOT NULL  GENERATED ALWAYS AS IDENTITY (START WITH 1, INCREMENT BY 1),
  tx_date   TIMESTAMP           NOT NULL  DEFAULT CURRENT_TIMESTAMP,
  tx_type   VARCHAR(10)         NOT NULL,
  item_name VARCHAR(100)        NOT NULL,
  user_name VARCHAR(100)        DEFAULT 'unknown',
  quantity  INT                 NOT NULL  DEFAULT 0,
  PRIMARY KEY  (id)
);

CREATE TABLE items (
  id            INT          NOT NULL  GENERATED ALWAYS AS IDENTITY (START WITH 1, INCREMENT BY 1),
  item_name     VARCHAR(100) NOT NULL,
  supplied_by   VARCHAR(100) NOT NULL  DEFAULT 'unknown',
  curr_stock    INT          NOT NULL  DEFAULT 0,
  item_price    FLOAT(2)     NOT NULL  DEFAULT 0.0,
  last_update   TIMESTAMP    NOT NULL  DEFAULT CURRENT_TIMESTAMP,
  PRIMARY KEY (id),
  UNIQUE (item_name)
);

INSERT INTO ITEMS (item_name, supplied_by, curr_stock, item_price) VALUES ('SOAP POWDER',   'Good Life Supplier',  2100, 2.50);
INSERT INTO ITEMS (item_name, supplied_by, curr_stock, item_price) VALUES ('DOG FOOD',      'Cool Stuff Supplier', 1400, 17.80);
INSERT INTO ITEMS (item_name, supplied_by, curr_stock, item_price) VALUES ('KITCHEN TOWEL', 'Cool Stuff Supplier', 1600, 8.00);
INSERT INTO ITEMS (item_name, supplied_by, curr_stock, item_price) VALUES ('LOTSA BREAD',   'Best Goods Supplier', 1950, 7.99);
INSERT INTO ITEMS (item_name, supplied_by, curr_stock, item_price) VALUES ('FISHY FINGERS', 'Good Life Supplier',  2700, 11.01);
INSERT INTO ITEMS (item_name, supplied_by, curr_stock, item_price) VALUES ('BAKED BEANS',   'Sweet Food Supplier', 3300, 10.99);
INSERT INTO ITEMS (item_name, supplied_by, curr_stock, item_price) VALUES ('WOA MILK',      'Good Life Supplier',  1000, 5.75);

CREATE TRIGGER trans_trg AFTER INSERT ON transactions
REFERENCING NEW as INSERTED_ROW
FOR EACH ROW MODE DB2SQL  
    UPDATE items SET
        curr_stock = CASE WHEN INSERTED_ROW.tx_type='buy' THEN curr_stock - INSERTED_ROW.quantity ELSE curr_stock + INSERTED_ROW.quantity END,
        last_update = CURRENT_TIMESTAMP
    WHERE
        INSERTED_ROW.item_name=item_name;
        