var itemTable = {
    
    /*
     * Add an item on a per row basis.
     * This is specifically for items table where the id of the tbody tag is "itemtbody"
     * and the header contains item, quantity, prices, and cost.
     */
    addItem: function(name,price,quantity) {
      
      var tbody = $("itemtbody");
      
      if(!tbody) return;
      
      //create stock row
      var row = document.createElement("tr");
      row.setAttribute("id", name);
      row.pfCells=new Array();

      // add stock cell
      var cell = document.createElement("td");
      var text=document.createTextNode(name);
      cell.className='tablerow';
      cell.setAttribute("bgcolor","#ffffff");
      cell.appendChild(text);
      row.appendChild(cell);
      
      // add value cells
      itemTable.addCell(row,'quantity',quantity);
      itemTable.addCell(row,'price','$'+itemTable.fixedDigits((price),2));
      itemTable.addCell(row,'cost','$'+itemTable.fixedDigits((quantity*price),2));
      
      //append row to table
      tbody.appendChild(row);
    },
    
    /*
     * Add the entire response in the table. Where the response object contains multiple items.
     * The response should contain the xml format
     * <sqlResult>
     *   <resultSet>
     *     <row item_name=".." etc.. />
     *   </resultSet>
     * </sqlResult>
     */
    addResponse: function(message){
     
      var resultSet = message.getElementsByTagName("resultSet")[0];
      var rows = resultSet.getElementsByTagName("row");
      
      var name = null;
      var quantity = null;
      var price = null;
      
      var item = null;
      var row = null;
      
      for(var i=0; i < rows.length; i++){
          item = rows[i];
          
          name = item.getAttribute('item_name');
          quantity = item.getAttribute('curr_stock');
          price = item.getAttribute('item_price');
          
          row = $(name);
      
          if (row)
          {
            if (quantity)
              itemTable.updateCell(row,'quantity',quantity);
        
            if (price)
              itemTable.updateCell(row,'price','$'+itemTable.fixedDigits(price,2));
        
            //add total cost by setting the product of price and quantity.
            itemTable.updateCell(row,'cost','$'+itemTable.fixedDigits((quantity*price),2));
      
          }else{
            //Add the row if it doesn't exist.
            itemTable.addItem(name,price,quantity);
          }   
      }         
    },
    
    /*
     * Updates an existing cell in the table.
     */
    updateCell: function(row,type,value) {
      
      if (row.pfCells[type]){
      
        var cell = row.pfCells[type];

        if (cell!=null && cell.lastValue!=value){
          cell.lastValue=value;
          cell.innerHTML=value;
        }
      }
    },
    
    /*
     * Add new cell in the given row.
     */
    addCell: function(row,type,value){
      
      var cell=document.createElement("td");
      var text=document.createTextNode(value);
      cell.className='tablerow';
      cell.setAttribute("bgcolor","#ffffff");
      cell.appendChild(text);
      row.appendChild(cell);
      row.pfCells[type]=cell;
      return cell;
    },
    
    /*
     * Converts number value to two digit decimal number.
     * Normally used for Prices and Cost.
     */
    fixedDigits: function(v, digits){
      var val=1*v;
      return (val.toFixed) ? val.toFixed(digits) : val;
    },
    
    removeRows: function(tbody, keepRows){
      var excludedIds = new Array("titleRow","headerRow");
      var rowCount = tbody.rows.length;
      
      if(!tbody || rowCount <= 2) return false;
      
      var elemToDelete = new Array();
      var ctr = 0;
      
      for(var i=0;i<=rowCount-1;i++){
        var rowElem = tbody.rows[i];
        var rowId = rowElem.getAttribute("id"); 
        var retainElem = "false";
        
        if((rowId == "titleRow") || (rowId == "headerRow")){
          retainElem = "true";
        } else {          
          var skip = "false";
          
          for(var j=0;j<keepRows.length;j++){
            if(rowId == keepRows[j] ){
              skip = "true";
            }
          }
          
          if(skip == "true"){
            retainElem = "true";
          }
        }
        
        if(retainElem == "false"){
          elemToDelete[ctr] = rowElem;
          ctr++;
        }
      }
      
      for(var k=0;k<elemToDelete.length;k++){
          var elem = elemToDelete[k];
          tbody.removeChild(elem); 
      }
    } 
};