/**
 *
 * Copyright 2005 LogicBlaze, Inc. http://www.logicblaze.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 **/
package org.logicblaze.soa.grocery.components;

import org.apache.servicemix.MessageExchangeListener;
import org.apache.servicemix.components.util.PollingComponentSupport;
import org.apache.servicemix.jbi.jaxp.StringSource;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.ExchangeStatus;
import javax.xml.transform.Source;
import java.sql.Timestamp;

public class InventoryDbPoller extends PollingComponentSupport implements MessageExchangeListener {
    private static final Log log = LogFactory.getLog(InventoryDbPoller.class);

    private Timestamp lastUpdate = new Timestamp(0);
    private boolean error = false;

    public void poll() {
        log.debug("InventoryDbPoller performing poll.");
        Timestamp newTimestamp = new Timestamp(System.currentTimeMillis());
        try {
            InOnly exchange = getExchangeFactory().createInOnlyExchange();
            NormalizedMessage message = exchange.createMessage();
            message.setContent(createSqlQuerySource(
                    "SELECT item_name, curr_stock, item_price " +
                    "FROM items WHERE last_update > '" + lastUpdate.toString() + "'"));
            exchange.setInMessage(message);
            send(exchange);
            lastUpdate = newTimestamp;
            error = false;
        } catch (Exception e) {
            if (!error) {
                log.error(e.getMessage(), e);
                error = true;
            }
        }
    }

    public Source createSqlQuerySource(String query) {
        return new StringSource("<sql>" + query + "</sql>");
    }

    public void onMessageExchange(MessageExchange exchange) throws MessagingException {
        // Ignore any done exchanges
        if (exchange.getStatus() == ExchangeStatus.DONE)
            return;

        // Ignore any error exchanges
        if (exchange.getStatus() == ExchangeStatus.ERROR) {
            return;
        }

        // Acknowledge any request
        this.done(exchange);
    }
}
