/**
 *
 * Copyright 2005 LogicBlaze, Inc. http://www.logicblaze.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 **/

package org.logicblaze.soa.components;

import org.apache.ddlutils.Platform;
import org.apache.ddlutils.PlatformFactory;
import org.apache.servicemix.jbi.util.DOMUtil;
import org.logicblaze.soa.util.ServletUtilities;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;
import java.io.IOException;
import java.io.PrintWriter;
import java.sql.Connection;

public class AjaxItemsServlet extends HttpServlet {

    private final static String CONTENT_TYPE = "text/xml";

    public void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        response.setContentType(CONTENT_TYPE);

        DataSource dataSource;
        try {
            dataSource = (DataSource) new InitialContext().lookup("java:comp/env/jdbc/inventoryDB");

        } catch (NamingException e) {
            throw new ServletException(e);
        }

        Platform platform = PlatformFactory.createNewPlatformInstance(dataSource);
        Connection connection = platform.borrowConnection();

        PrintWriter writer = response.getWriter();

        String strXMLResponse = null;
        try {
            strXMLResponse = DOMUtil.asXML(ServletUtilities.createXMLMessage(connection));
        } catch (ParserConfigurationException e) {
            throw new IOException("Parser Configuration error " + e);

        } catch (TransformerException e) {
            throw new IOException("Transformer error " + e);

        }

        writer.print(strXMLResponse);

        platform.returnConnection(connection);

        writer.close();
    }

    /**
     * Forward the request to doGet method.
     *
     * @param request
     * @param response
     * @throws ServletException
     * @throws IOException
     */
    public void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        doGet(request, response);

    }
}
