/**
 *
 * Copyright 2005 LogicBlaze, Inc. http://www.logicblaze.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 **/

package org.logicblaze.soa.components;

import org.apache.ddlutils.Platform;
import org.apache.ddlutils.PlatformFactory;
import org.logicblaze.soa.util.ServletUtilities;

import javax.naming.InitialContext;
import javax.servlet.ServletException;
import javax.servlet.RequestDispatcher;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;
import java.io.IOException;
import java.sql.Connection;

public class ItemsServlet extends HttpServlet {

    /**
     * Process the request.
     * @param request
     * @param response
     * @throws ServletException
     * @throws IOException
     */
    public void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        try {
            DataSource  dataSource = (DataSource) new InitialContext().lookup("java:comp/env/jdbc/inventoryDB");
            Platform platform = PlatformFactory.createNewPlatformInstance(dataSource);
            Connection connection = platform.borrowConnection();

            request.setAttribute("itemListBean",ServletUtilities.createItemListMessage(connection));
            platform.returnConnection(connection);
            forward(request,response, "/ItemMonitorJSP");

        } catch (Exception e) {
            throw new ServletException(e);

        }
    }

    /**
     * Forward the request to doGet method.
     * @param request
     * @param response
     * @throws ServletException
     * @throws IOException
     */
    public void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        doGet(request,response);
    }

    /**
     * Creates a request dispatcher object to forward the response to a jsp.
     * @param request
     * @param response
     * @param url
     * @throws IOException
     * @throws ServletException
     */
    public void forward(HttpServletRequest request, HttpServletResponse response, String url)
            throws IOException, ServletException {

        RequestDispatcher rd = getServletContext().getRequestDispatcher(url);
        if (rd == null) {
            throw new ServletException("Could not get RequestDispatcher for " + url);
        }
        rd.forward(request, response);
    }
}