/**
 *
 * Copyright 2005 LogicBlaze, Inc. http://www.logicblaze.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 **/
package org.logicblaze.soa.components;

import org.logicblaze.soa.util.ServletUtilities;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;

public class TransactionServlet extends HttpServlet {

    private static final Log log = LogFactory.getLog(TransactionServlet.class);

    private final static String CONTENT_TYPE = "text/xml";
    private final static String CHARSET = "utf-8";

    /**
     * Receives and processes all the request.
     *
     * @param request
     * @param response
     * @throws ServletException
     * @throws IOException
     */
    public void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        response.setContentType(CONTENT_TYPE);

        String url = request.getParameter("url");
        String content = request.getParameter("content");
        String number = request.getParameter("number");
        String interval = request.getParameter("interval");

        PrintWriter writer = response.getWriter();

        if (number != null) {
            int msgCount = Integer.parseInt(number);
            int msgInterval = Integer.parseInt(interval);
            int updates = sendRandomTransactions(url, msgCount, msgInterval);
            writer.print("<sqlResult><updateCount value='" + updates + "'/></sqlResult>");
        } else {
            writer.print(ServletUtilities.sendRequest(url, content, CONTENT_TYPE, CHARSET));
        }

        writer.close();
        return;
    }

    /**
     * Forwards the request to Get
     *
     * @param request
     * @param response
     * @throws ServletException
     * @throws IOException
     */
    public void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        doGet(request, response);

    }

    public int sendRandomTransactions(String url, int numberOfTransactions, int interval) throws IOException {

        int ctr = 0;
        for (int i = 0; i < numberOfTransactions; i++) {
            String content = generateRandomContent(url);
            ServletUtilities.sendRequest(url, content, CONTENT_TYPE, CHARSET);

            log.info(i+ ":" + content);

            ctr++;
            try {
                Thread.sleep(interval * 1000);
            } catch (InterruptedException e) {
                System.err.println("InterruptedException occured" + e.getMessage());
            }
        }
        return ctr;
    }

    public String generateRandomContent(String url) throws IOException {
        StringBuffer buffer = new StringBuffer();
        buffer.append("<sql>INSERT INTO transactions (tx_type, item_name, user_name, quantity) values ('");

        String type = getTransaction();
        String user;

        if (type.equals("buy")) {
            user = getConsumer();
        } else {
            user = getSupplier();
        }

        buffer.append(type + "','");
        buffer.append(getItem() + "','");
        buffer.append(user + "',");
        buffer.append(getQuantity());
        buffer.append(")</sql>");

        return buffer.toString();
    }


    // Helper methods that generated random values.

    private String getSupplier() {
        String suppliers[] = {"Good Life Supplier", "Cool Stuff Supplier", "Best Goods Supplier", "Sweet Food Supplier"};
        return suppliers[(int)(Math.random() * 4)];
    }

    private String getConsumer() {
        String consumers[] = {"Joe Smith", "Juan Dela Cruz", "Charles Heynes",
                              "Bob Chap", "Kobe Bryant", "Lamar Odom", "Phil Jackson",
                              "Robert Joe", "Pards Peterson", "Amiel Osama"};
        return consumers[(int)(Math.random() * 10)];
    }

    private String getTransaction() {
        String transactions[] = {"buy", "sell"};
        return transactions[(int)(Math.random() * 2)];
    }

    private String getItem() {
        String items[] = {"SOAP POWDER", "DOG FOOD", "KITCHEN TOWEL", "LOTSA BREAD", "FISHY FINGERS", "BAKED BEANS", "WOA MILK"};
        return items[(int)(Math.random() * 7)];
    }

    private int getQuantity() {
        return (int)(Math.random() * 100);
    }
}



