/**
 *
 * Copyright 2005 LogicBlaze, Inc. http://www.logicblaze.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 **/
package org.logicblaze.soa.util;

import org.apache.commons.httpclient.DefaultHttpMethodRetryHandler;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.StringRequestEntity;
import org.apache.commons.httpclient.params.HttpMethodParams;
import org.apache.servicemix.jbi.jaxp.SourceTransformer;
import org.w3c.dom.Element;
import org.w3c.dom.Document;

import javax.xml.parsers.ParserConfigurationException;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.sql.*;
import java.util.ArrayList;
import java.util.List;

public class ServletUtilities {

    /**
     * Transform the data queried from the database into an xml string format. This is used by the
     * ajax item monitoring demo.
     *
     * @param connection
     * @return
     */
    public static Element createXMLMessage(Connection connection) throws ParserConfigurationException {

        SourceTransformer domTransform = new SourceTransformer();
        Document document = domTransform.createDocument();

        Element elemSQL = document.createElement("sqlResult");
        Element elemRS = document.createElement("resultSet");

        Statement stmt = null;
        ResultSet srs = null;

        try {
            stmt = connection.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
            srs = stmt.executeQuery("select * from items");

            //Store the data in an xml format.
            elemRS.setAttribute("status", "success");

            ResultSetMetaData meta = srs.getMetaData();
            int colCount = meta.getColumnCount();

            Element elemItem;
            while (srs.next()) {
                elemItem = document.createElement("row");
                for (int i = 1; i <= colCount; i++) {
                    String colName = meta.getColumnName(i);
                    if (colName.equals("")) {
                        continue;
                    }
                    elemItem.setAttribute(colName.toLowerCase(), srs.getString(i));
                }
                elemRS.appendChild(elemItem);
            }

            elemSQL.appendChild(elemRS);

            stmt.close();
            srs.close();

        } catch (SQLException e) {
            Element elemException = document.createElement("exception");
            elemException.setAttribute("class", e.getClass().toString());
            elemException.setNodeValue(e.getMessage());

            elemSQL.appendChild(elemException);
        }

        return elemSQL;
    }

    /**
     * Creates an ItemListBean object to be passed to the item monitor jsp. This is used by the
     * typical servlet/jsp monitoring demo.
     *
     * @param connection
     * @return
     * @throws SQLException
     */
    public static ItemListBean createItemListMessage(Connection connection) throws SQLException {

        Statement stmt = connection.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
        ResultSet srs = stmt.executeQuery("SELECT * FROM items");

        //Store the data in an xml format.
        List list = new ArrayList();
        ItemContentBean itemBean = null;

        ResultSetMetaData meta = srs.getMetaData();
        int colCount = meta.getColumnCount();

        while (srs.next()) {
            itemBean = new ItemContentBean();
            for (int i = 1; i <= colCount; i++) {

                String colName = meta.getColumnName(i);

                if ("item_name".equalsIgnoreCase(colName)) {
                    itemBean.setName(srs.getString(i));

                } else if ("item_price".equalsIgnoreCase(colName)) {
                    itemBean.setPrice(srs.getString(i));

                } else if ("curr_stock".equalsIgnoreCase(colName)) {
                    itemBean.setQuantity(srs.getInt(i));

                } else if ("last_update".equalsIgnoreCase(colName)) {
                    itemBean.setLastUpdate(srs.getTimestamp(i));

                } else {
                    continue;

                }
            }
            list.add(itemBean);
        }

        stmt.close();
        srs.close();

        return new ItemListBean(list);
    }

    /**
     * Method use to send http request.
     *
     * @param url         - url location of the http endpoint
     * @param content
     * @param contentType - type of request content sent (e.g. text/xml)
     * @param charset     - type of charset (e.g. ISO-8859-1)
     * @return
     */
    public static String sendRequest(String url, String content, String contentType, String charset)
            throws IOException {
        // Create an instance of HttpClient.
        HttpClient client = new HttpClient();
        // Create a method instance.
        PostMethod method = new PostMethod(url);

        String strResponse = null;
        try {
            method.setRequestEntity(new StringRequestEntity(content, contentType, charset));
            // Provide custom retry handler is necessary
            method.getParams().setParameter(HttpMethodParams.RETRY_HANDLER, new DefaultHttpMethodRetryHandler(3, false));
            // Execute the method.
            int statusCode = client.executeMethod(method);

            if (statusCode != HttpStatus.SC_OK) {
                throw new IOException("Method failed: " + method.getStatusLine());

            }

            // Read the response body.
            InputStream is = method.getResponseBodyAsStream();
            strResponse = convertStreamToString(is);

        } catch (HttpException e) {
            throw new IOException("Fatal protocol violation" + e);

        } catch (IOException e) {
            throw new IOException("Fatal transport error: " + e);

        } finally {
            // Release the connection.
            method.releaseConnection();

        }

        return strResponse;
    }

    /**
     * Converts InputStream to String.
     * @param is
     * @return
     * @throws IOException
     */
    public static String convertStreamToString(InputStream is) throws IOException {
        String line = null;
        BufferedReader in = new BufferedReader(new InputStreamReader(is));
        StringBuffer buffer = new StringBuffer();
        while ((line = in.readLine()) != null) {
            buffer.append(line);
        }
        //return new BufferedReader(new StringReader(buffer.toString()));
        return buffer.toString();
    }
}