package loanbroker;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.namespace.QName;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathFactory;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.servicemix.MessageExchangeListener;
import org.apache.servicemix.components.util.ComponentSupport;
import org.apache.servicemix.jbi.jaxp.SourceTransformer;
import org.apache.servicemix.jbi.jaxp.StringSource;
import org.w3c.dom.Node;

public class Bank0 extends ComponentSupport implements MessageExchangeListener {
    
    private static final Log log = LogFactory.getLog(Bank0.class);
    
    public void onMessageExchange(MessageExchange exchange) throws MessagingException {
        if (exchange.getStatus() == ExchangeStatus.DONE) {
            return;
        } else if (exchange.getStatus() == ExchangeStatus.ERROR) {
            return;
        }
        try {
            Thread.sleep((long) (Math.random() * 5000));
            Node doc = new SourceTransformer().toDOMNode(exchange.getMessage("in"));
            String id = textValueOfXPath(doc, "//*[local-name()='id']");
            String bestBank = textValueOfXPath(doc, "//*[local-name()='bank']");
            String bestRate = textValueOfXPath(doc, "//*[local-name()='rate']");
            // Do not compete against ourself
            if ("Bank0".equals(bestBank)) {
                return;
            }
            double rate = (Math.ceil(1000 * Math.random()) / 100);
            if (bestRate != null && bestRate.length() > 0) {
                double bRate = Double.parseDouble(bestRate);
                // The new rate is not better than the best one
                if (rate >= bRate) {
                    return;
                }
            }
            log.info("Bank0 requested: " + rate + " for id: " + id);
            InOnly out = createInOnlyExchange(new QName("urn:logicblaze:soa:loanbroker", "BankQuoteCallbackService"), null, null);
            out.setOperation(new QName("urn:logicblaze:soa:loanbroker", "getLoanQuoteCallback"));
            String output = "<getLoanQuoteResponse xmlns=\"urn:logicblaze:soa:bank\"><id>" + id + "</id><bank>Bank0</bank><rate>" + rate + "</rate></getLoanQuoteResponse>";
            NormalizedMessage answer = out.createMessage();
            answer.setContent(new StringSource(output));
            out.setInMessage(answer);
            send(out);
        } catch (Exception e) {
            throw new MessagingException(e);
        } finally {
            done(exchange);
        }
    }

    protected static String textValueOfXPath(Node node, String exp) throws Exception {
        XPath xpath = XPathFactory.newInstance().newXPath();
        return xpath.evaluate(exp, node);
    }
}

