import java.io.File;
import java.net.URI;
import java.net.URL;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.naming.Context;
import javax.xml.namespace.QName;
import javax.xml.transform.TransformerException;
import javax.xml.transform.dom.DOMSource;

import junit.framework.TestCase;

import org.apache.activemq.ActiveMQConnectionFactory;
import org.apache.servicemix.MessageExchangeListener;
import org.apache.servicemix.bpe.BPEComponent;
import org.apache.servicemix.client.DefaultServiceMixClient;
import org.apache.servicemix.client.ServiceMixClient;
import org.apache.servicemix.components.util.ComponentSupport;
import org.apache.servicemix.jbi.container.ActivationSpec;
import org.apache.servicemix.jbi.container.JBIContainer;
import org.apache.servicemix.jbi.jaxp.SourceTransformer;
import org.apache.servicemix.jbi.jaxp.StringSource;
import org.apache.servicemix.jbi.util.DOMUtil;
import org.apache.servicemix.tck.ReceiverComponent;
import org.apache.servicemix.wsn.client.NotificationBroker;
import org.apache.servicemix.wsn.component.WSNComponent;
import org.apache.xbean.spring.jndi.SpringInitialContextFactory;
import org.w3._2005._08.addressing.AttributedURIType;
import org.w3._2005._08.addressing.EndpointReferenceType;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.traversal.NodeIterator;

import com.sun.org.apache.xpath.internal.CachedXPathAPI;

public class TestLoanBroker extends TestCase {

    private JBIContainer jbi;
    
    protected void setUp() throws Exception {
        System.setProperty(Context.INITIAL_CONTEXT_FACTORY, "org.apache.xbean.spring.jndi.SpringInitialContextFactory");
        SpringInitialContextFactory.makeInitialContext();
    }
    
    protected void shutDown() throws Exception {
        jbi.shutDown();
    }
    
    public void testLB() throws Exception {
        jbi = new JBIContainer();
        jbi.setEmbedded(true);
        jbi.init();
        jbi.start();
        
        ReceiverComponent receiver = new ReceiverComponent();
        ActivationSpec asReceiver = new ActivationSpec();
        asReceiver.setComponent(receiver);
        asReceiver.setInterfaceName(new QName("urn:logicblaze:soa:loanbroker", "LoanBrokerAsyncCallback"));
        jbi.activateComponent(asReceiver);
        
        registerWSNBroker();
        registerCreditAgency();
        registerBanks();
        BPEComponent bpe = new BPEComponent();
        jbi.activateComponent(bpe, "bpe");
        
        URL url = getClass().getClassLoader().getResource("loanbroker.bpel");
        File path = new File(new URI(url.toString()));
        path = path.getParentFile();
        bpe.getServiceUnitManager().deploy("loanbroker-async", path.getAbsolutePath());
        bpe.getServiceUnitManager().start("loanbroker-async");
        
        ServiceMixClient client = new DefaultServiceMixClient(jbi);
        //MessageExchange me = client.createInOutExchange();
        
        int nb = 10;
        MessageExchange[] mes = new MessageExchange[nb];
        for (int i = 0; i < mes.length; i++) {
            mes[i] = client.createInOnlyExchange();
            mes[i].setService(new QName("urn:logicblaze:soa:loanbroker", "LoanBrokerAsyncService"));
            mes[i].setOperation(new QName("getLoanQuote"));
            mes[i].getMessage("in").setContent(new StringSource("<getLoanQuoteRequest xmlns=\"urn:logicblaze:soa:loanbroker\"><id>" + i + "</id><ssn>1234341</ssn><amount>100000.0</amount><duration>12</duration></getLoanQuoteRequest>"));
        }
        for (int i = 0; i < mes.length; i++) {
            client.send(mes[i]);
        }
        
        receiver.getMessageList().waitForMessagesToArrive(nb, 30000);
        assertEquals("expected number of messages when received: " + receiver.getMessageList().getMessages(), nb, receiver.getMessageList().getMessageCount());
        List messages = receiver.getMessageList().getMessages();
        for (Iterator iter = messages.iterator(); iter.hasNext();) {
            NormalizedMessage msg = (NormalizedMessage) iter.next();
            System.err.println(new SourceTransformer().toString(msg.getContent()));
        }
    }
    
    public void testLBSync() throws Exception {
        jbi = new JBIContainer();
        jbi.setEmbedded(true);
        jbi.init();
        jbi.start();
        
        registerWSNBroker();
        registerCreditAgency();
        registerLoanBroker();
        registerBanks();
        BPEComponent bpe = new BPEComponent();
        jbi.activateComponent(bpe, "bpe");
        
        URL url = getClass().getClassLoader().getResource("loanbroker.bpel");
        File path = new File(new URI(url.toString()));
        path = path.getParentFile();
        bpe.getServiceUnitManager().deploy("loanbroker-async", path.getAbsolutePath());
        bpe.getServiceUnitManager().start("loanbroker-async");
        
        DefaultServiceMixClient client = new DefaultServiceMixClient(jbi);
        
        int nb = 10;
        for (int i = 0; i < nb; i++) {
            MessageExchange me = client.createInOutExchange();
            me.setService(new QName("urn:logicblaze:soa:loanbroker", "LoanBrokerService"));
            me.setOperation(new QName("getLoanQuote"));
            me.getMessage("in").setContent(new StringSource("<getLoanQuoteRequest xmlns=\"urn:logicblaze:soa:loanbroker\"><ssn>1234341</ssn><amount>100000.0</amount><duration>12</duration></getLoanQuoteRequest>"));
            client.sendSync(me);
            NormalizedMessage msg = me.getMessage("out");
            System.err.println(new SourceTransformer().toString(msg.getContent()));
        }
    }
    
    protected void registerWSNBroker() throws Exception {
        ActivationSpec wsnBroker = new ActivationSpec();
        WSNComponent wsnComponent = new WSNComponent();
        wsnComponent.setConnectionFactory(new ActiveMQConnectionFactory("vm://localhost?broker.persistent=false"));
        wsnBroker.setComponent(wsnComponent);
        jbi.activateComponent(wsnBroker);
    }

    protected void registerCreditAgency() throws Exception {
        ActivationSpec creditAgency = new ActivationSpec();
        creditAgency.setInterfaceName(new QName("urn:logicblaze:soa:creditagency", "CreditAgency"));
        creditAgency.setComponent(new CreditAgency());
        jbi.activateComponent(creditAgency);
    }
    
    protected void registerLoanBroker() throws Exception {
        ActivationSpec lb = new ActivationSpec();
        lb.setComponent(new LoanBroker());
        jbi.activateComponent(lb);
    }
    
    protected void registerBanks() throws Exception {
        for (int i = 1; i <= 3; i++) {
            ActivationSpec bank = new ActivationSpec();
            bank.setInterfaceName(new QName("urn:logicblaze:soa:bank", "Bank"));
            Bank bk = new Bank(i);
            bank.setComponent(bk);
            jbi.activateComponent(bank);
            EndpointReferenceType epr = createEPR(bk.getService(), bk.getEndpoint());
            new NotificationBroker(jbi).subscribe(epr, "logicblaze.soa.loanbroker.request");
        }
    }
    
    protected EndpointReferenceType createEPR(QName service, String endpoint) {
        EndpointReferenceType epr = new EndpointReferenceType();
        epr.setAddress(new AttributedURIType());
        epr.getAddress().setValue(service.getNamespaceURI() + "/" + service.getLocalPart() + "/" + endpoint);
        return epr;
    }
    
    public static class LoanBroker extends ComponentSupport implements MessageExchangeListener {
        
        private Map exchanges = new ConcurrentHashMap();
        
        public LoanBroker() {
        }
        
        protected void init() throws JBIException {
            super.init();
            ComponentContext context = getContext();
            context.activateEndpoint(new QName("urn:logicblaze:soa:loanbroker", "LoanBrokerService"),
                                     "loanbroker");
            context.activateEndpoint(new QName("urn:logicblaze:soa:loanbroker", "LoanBrokerAsyncCallbackService"),
                                     "loanbroker");
        }
        
        public void onMessageExchange(MessageExchange exchange) throws MessagingException {
            if (exchange.getStatus() == ExchangeStatus.DONE) {
                return;
            } else if (exchange.getStatus() == ExchangeStatus.ERROR) {
                done(exchange);
                return;
            }
            try {
                String service = exchange.getEndpoint().getServiceName().getLocalPart();
                if ("LoanBrokerService".equals(service)) {
                    String id = exchange.getExchangeId();
                    Node doc = new SourceTransformer().toDOMNode(exchange.getMessage("in"));
                    Element element = (doc instanceof Document) ? ((Document) doc).getDocumentElement() : (Element) doc;
                    Document document = element.getOwnerDocument();
                    Element child = document.createElementNS("urn:logicblaze:soa:loanbroker", "lb:id");
                    element.appendChild(child);
                    child.appendChild(document.createTextNode(id));
                    InOnly request = createInOnlyExchange(new QName("urn:logicblaze:soa:loanbroker", "LoanBrokerAsyncService"),
                                                          new QName("urn:logicblaze:soa:loanbroker", "LoanBrokerAsync"), 
                                                          null); 
                    request.setOperation(new QName("urn:logicblaze:soa:loanbroker", "getLoanQuote"));
                    NormalizedMessage in = request.createMessage();
                    in.setContent(new DOMSource(doc));
                    request.setInMessage(in);
                    send(request);
                    exchanges.put(exchange.getExchangeId(), exchange);
                } else if ("LoanBrokerAsyncCallbackService".equals(service)) {
                    Node doc = new SourceTransformer().toDOMNode(exchange.getMessage("in"));
                    String id = textValueOfXPath(doc, "//*[local-name()='id']");
                    MessageExchange request = (MessageExchange) exchanges.get(id); 
                    NormalizedMessage out = request.createMessage();
                    out.setContent(new DOMSource(doc));
                    answer(request, out);
                }
            } catch (Exception e) {
                throw new MessagingException(e);
            }
        }
    }
    
    public static class Bank extends ComponentSupport implements MessageExchangeListener {
        
        public Bank(int number) {
            setService(new QName("urn:logicblaze:soa:bank", "Bank" + number));
            setEndpoint("bank");
        }
        
        public void onMessageExchange(MessageExchange exchange) throws MessagingException {
            if (exchange.getStatus() == ExchangeStatus.DONE) {
                return;
            } else if (exchange.getStatus() == ExchangeStatus.ERROR) {
                done(exchange);
                return;
            }
            try {
                Node doc = new SourceTransformer().toDOMNode(exchange.getMessage("in"));
                String id = textValueOfXPath(doc, "//*[local-name()='id']");
                double rate = (Math.ceil(1000 * Math.random()) / 100);
                System.err.println(getService().getLocalPart() + " requested: " + rate);
                InOnly out = createInOnlyExchange(new QName("urn:logicblaze:soa:loanbroker", "BankQuoteCallbackService"), null, null);
                out.setOperation(new QName("urn:logicblaze:soa:bank", "getLoanQuoteCallback"));
                String output = "<getLoanQuoteResponse xmlns=\"urn:logicblaze:soa:bank\"><id>" + id + "</id><bank>" + getService().getLocalPart() + "</bank><rate>" + rate + "</rate></getLoanQuoteResponse>";
                NormalizedMessage answer = out.createMessage();
                answer.setContent(new StringSource(output));
                out.setInMessage(answer);
                send(out);
                done(exchange);
            } catch (Exception e) {
                throw new MessagingException(e);
            }
        }
    }
    
    public static class CreditAgency extends ComponentSupport implements MessageExchangeListener {

        public CreditAgency() {
            setService(new QName("urn:logicblaze:soa:creditagency", "CreditAgencyService"));
            setEndpoint("agency");
        }
        
        public void onMessageExchange(MessageExchange exchange) throws MessagingException {
            InOut inOut = (InOut) exchange;
            if (inOut.getStatus() == ExchangeStatus.DONE) {
                return;
            } else if (inOut.getStatus() == ExchangeStatus.ERROR) {
                done(inOut);
                return;
            }
            try {
                Document doc = (Document) new SourceTransformer().toDOMNode(inOut.getInMessage());
                String ssn = textValueOfXPath(doc, "//*[local-name()='ssn']");
                if (ssn == null || ssn.length() == 0) {
                    fail(exchange, new NullPointerException());
                    return;
                } 
                if (!ssn.startsWith("1")) {
                    Fault fault = inOut.createFault();
                    fault.setContent(new StringSource("<InvalidSSN> xmlns=\"urn:logicblaze:soa:creditagency\"<ssn>" + ssn + "</ssn></InvalidSSN>"));
                    fail(inOut, fault);
                } else {
                    String operation = null;
                    if (inOut.getOperation() != null) {
                        operation = inOut.getOperation().getLocalPart();
                    } else {
                        operation = doc.getDocumentElement().getLocalName();
                    }
                    String output;
                    if ("getCreditScore".equals(operation)) {
                        output = "<getCreditScoreResponse xmlns=\"urn:logicblaze:soa:creditagency\"><score>" + getCreditScore(ssn) + "</score></getCreditScoreResponse>";
                    } else if ("getCreditHistoryLength".equals(operation)) {
                        output = "<getCreditHistoryLengthResponse xmlns=\"urn:logicblaze:soa:creditagency\"><length>" + getCreditHistoryLength(ssn) + "</length></getCreditHistoryLengthResponse>";
                    } else {
                        throw new UnsupportedOperationException(operation);
                    }
                    NormalizedMessage answer = inOut.createMessage();
                    answer.setContent(new StringSource(output));
                    answer(inOut, answer);
                }
            } catch (Exception e) {
                throw new MessagingException(e);
            }
        }
        int getCreditScore(String ssn) {
            //return ((int) (Math.random() * 600) + 300);
            return 1000;
        }
        int getCreditHistoryLength(String ssn) {
            //return ((int) (Math.random() * 19) + 1);
            return 10;
        }
        
    }
    
    protected static String textValueOfXPath(Node node, String xpath) throws TransformerException {
        CachedXPathAPI cachedXPathAPI = new CachedXPathAPI();
        NodeIterator iterator = cachedXPathAPI.selectNodeIterator(node, xpath);
        Node root = iterator.nextNode();
        if (root instanceof Element) {
            Element element = (Element) root;
            if (element == null) {
                return "";
            }
            String text = DOMUtil.getElementText(element);
            return text;
        }
        else if (root != null) {
            return root.getNodeValue();
        } else {
            return null;
        }
    }
}
