package loanbroker;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.namespace.QName;
import javax.xml.transform.dom.DOMSource;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathFactory;

import org.apache.servicemix.MessageExchangeListener;
import org.apache.servicemix.components.util.ComponentSupport;
import org.apache.servicemix.jbi.jaxp.SourceTransformer;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class LoanBroker extends ComponentSupport implements MessageExchangeListener {

    private Map exchanges = new ConcurrentHashMap();
    
    public LoanBroker() {
    }
    
    protected void init() throws JBIException {
        super.init();
        ComponentContext context = getContext();
        context.activateEndpoint(new QName("urn:logicblaze:soa:loanbroker", "LoanBrokerService"),
                                 "loanbroker");
        context.activateEndpoint(new QName("urn:logicblaze:soa:loanbroker", "LoanBrokerAsyncCallbackService"),
                                 "loanbroker");
    }
    
    public void onMessageExchange(MessageExchange exchange) throws MessagingException {
        if (exchange.getStatus() == ExchangeStatus.DONE) {
            return;
        } else if (exchange.getStatus() == ExchangeStatus.ERROR) {
            return;
        }
        try {
            String service = exchange.getEndpoint().getServiceName().getLocalPart();
            if ("LoanBrokerService".equals(service)) {
                String id = exchange.getExchangeId();
                Node doc = new SourceTransformer().toDOMNode(exchange.getMessage("in"));
                Element element = (doc instanceof Document) ? ((Document) doc).getDocumentElement() : (Element) doc;
                Document document = element.getOwnerDocument();
                Element child = document.createElementNS("urn:logicblaze:soa:loanbroker", "lb:id");
                element.appendChild(child);
                child.appendChild(document.createTextNode(id));
                InOnly request = createInOnlyExchange(new QName("urn:logicblaze:soa:loanbroker", "LoanBrokerAsyncService"),
                                                      new QName("urn:logicblaze:soa:loanbroker", "LoanBrokerAsync"), 
                                                      null); 
                request.setOperation(new QName("urn:logicblaze:soa:loanbroker", "getLoanQuote"));
                NormalizedMessage in = request.createMessage();
                in.setContent(new DOMSource(doc));
                request.setInMessage(in);
                send(request);
                exchanges.put(exchange.getExchangeId(), exchange);
            } else if ("LoanBrokerAsyncCallbackService".equals(service)) {
                Node doc = new SourceTransformer().toDOMNode(exchange.getMessage("in"));
                String id = textValueOfXPath(doc, "//*[local-name()='id']");
                MessageExchange request = (MessageExchange) exchanges.get(id); 
                NormalizedMessage out = request.createMessage();
                out.setContent(new DOMSource(doc));
                answer(request, out);
                done(exchange);
            } else {
                throw new UnsupportedOperationException("Unkown service: " + service);
            }
        } catch (Exception e) {
            throw new MessagingException(e);
        }
    }

    protected static String textValueOfXPath(Node node, String exp) throws Exception {
        XPath xpath = XPathFactory.newInstance().newXPath();
        return xpath.evaluate(exp, node);
    }
}
