/*
 * Copyright 2006 the Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.livetribe.webconsole.servicemix.command;

import java.util.ResourceBundle;
import java.util.MissingResourceException;
import java.text.MessageFormat;
import javax.portlet.PortletSession;

import org.livetribe.web.command.AbstractCommand;
import org.livetribe.web.command.portlet.PortletResourceBundleAware;
import org.livetribe.web.command.portlet.PortletSessionAware;

/**
 * @version $Rev: 113 $ $Date: 2006-03-11 01:22:32 +0800 (Sat, 11 Mar 2006) $
 */
public abstract class AbstractPortletCommand extends AbstractCommand implements PortletSessionAware, PortletResourceBundleAware
{
    protected static final String MESSAGES_KEY = "command.messages";
    private PortletSession portletSession;
    private ResourceBundle resourceBundle;

    public void setPortletSession(PortletSession session)
    {
        this.portletSession = session;
    }

    public void setResourceBundle(ResourceBundle resourceBundle)
    {
        this.resourceBundle = resourceBundle;
    }

    protected PortletSession getPortletSession()
    {
        return portletSession;
    }

    protected ResourceBundle getResourceBundle()
    {
        return resourceBundle;
    }

    protected String getMessage(String key)
    {
        try
        {
            return getResourceBundle().getString(key);
        }
        catch (MissingResourceException x)
        {
            return "???" + key + "???";
        }
    }

    protected String getMessage(String key, Object[] args)
    {
        String message = getMessage(key);
        return MessageFormat.format(message, args);
    }

    protected String getMessage(String key, String defaultMessage, Object[] args)
    {
        String message = null;
        try
        {
            message = getResourceBundle().getString(key);
        }
        catch (MissingResourceException x)
        {
            message = defaultMessage;
        }catch (NullPointerException npe)
        {
            message = defaultMessage;
        }
        
        if (args != null && args.length > 0)
        {
            return MessageFormat.format(message, args);
        }
        else
        {
            return message;
        }
    }

    public static class CommandMessage
    {
        public static final String INFO = "info";
        public static final String WARNING = "warning";
        public static final String ERROR = "error";

        private final String type;
        private final String text;

        public CommandMessage(String type, String text)
        {
            this.type = type;
            this.text = text;
        }

        public String getType()
        {
            return type;
        }

        public String getText()
        {
            return text;
        }
    }
}
